@@ -1,7 +1,7 @@
 /**
  * @file node_type.h  node type interface
  * 
- * Copyright (C) 2007-2008 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2012 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,20 +23,24 @@
 
 #include "node.h"
 
+#include <libxml/xmlmemory.h>
+#include <gtk/gtk.h>
+
 /** node type capabilities */
 enum {
 	NODE_CAPABILITY_SHOW_ITEM_FAVICONS	= (1<<0),	/**< display favicons in item list (useful for recursively viewed leaf node) */
 	NODE_CAPABILITY_ADD_CHILDS		= (1<<1),	/**< allows adding new childs */
 	NODE_CAPABILITY_REMOVE_CHILDS		= (1<<2),	/**< allows removing it's childs */
 	NODE_CAPABILITY_SUBFOLDERS		= (1<<3),	/**< allows creating/removing sub folders */
-	NODE_CAPABILITY_REMOVE_ITEMS		= (1<<4),	/**< allows removing of single items */
-	NODE_CAPABILITY_RECEIVE_ITEMS		= (1<<5),	/**< is a DnD target for item copies */
-	NODE_CAPABILITY_REORDER			= (1<<6),	/**< allows DnD to reorder childs */
-	NODE_CAPABILITY_SHOW_UNREAD_COUNT	= (1<<7),	/**< display the unread item count in the feed list */
-	NODE_CAPABILITY_SHOW_ITEM_COUNT		= (1<<8),	/**< display the absolute item count in the feed list */
-	NODE_CAPABILITY_UPDATE			= (1<<9),	/**< node type always has a subscription and can be updated */
-	NODE_CAPABILITY_UPDATE_CHILDS		= (1<<10),	/**< childs of this node type can be updated */
-	NODE_CAPABILITY_EXPORT			= (1<<11)	/**< nodes of this type can be exported safely to OPML */
+	NODE_CAPABILITY_REMOVE_ITEMS		= (1<<5),	/**< allows removing of single items */
+	NODE_CAPABILITY_RECEIVE_ITEMS		= (1<<6),	/**< is a DnD target for item copies */
+	NODE_CAPABILITY_REORDER			= (1<<7),	/**< allows DnD to reorder childs */
+	NODE_CAPABILITY_SHOW_UNREAD_COUNT	= (1<<8),	/**< display the unread item count in the feed list */
+	NODE_CAPABILITY_SHOW_ITEM_COUNT		= (1<<9),	/**< display the absolute item count in the feed list */
+	NODE_CAPABILITY_UPDATE			= (1<<10),	/**< node type always has a subscription and can be updated */
+	NODE_CAPABILITY_UPDATE_CHILDS		= (1<<11),	/**< childs of this node type can be updated */
+	NODE_CAPABILITY_UPDATE_FAVICON		= (1<<12),	/**< this node allows downloading a favicon */
+	NODE_CAPABILITY_EXPORT			= (1<<13)	/**< nodes of this type can be exported safely to OPML */
 };
 
 /**
@@ -49,7 +53,7 @@ enum {
 typedef struct nodeType {
 	gulong		capabilities;	/**< bitmask of node type capabilities */
 	const gchar	*id;		/**< type id (used for type attribute in OPML export) */
-	gpointer	icon;		/**< default icon */
+	const GdkPixbuf	*icon;		/**< default icon for nodes of this type (if no favicon available) */
 	
 	/* For method documentation see the wrappers defined below! 
 	   All methods are mandatory for each node type. */
@@ -82,7 +86,7 @@ typedef struct nodeType {
  *
  * @returns type string (or NULL if unknown)
  */
-const gchar *node_type_to_str(nodePtr node);
+const gchar *node_type_to_str (nodePtr node);
 
 /** 
  * Maps node type string to type constant.
@@ -91,7 +95,7 @@ const gchar *node_type_to_str(nodePtr node);
  *
  * @returns node type
  */
-nodeTypePtr node_str_to_type(const gchar *str);
+nodeTypePtr node_str_to_type (const gchar *str);
 
 /**
  * Interactive node adding (e.g. feed menu->new subscription), 
@@ -102,6 +106,6 @@ nodeTypePtr node_str_to_type(const gchar *str);
  *
  * @returns TRUE on success
  */
-gboolean node_type_request_interactive_add(nodeTypePtr nodeType);
+gboolean node_type_request_interactive_add (nodeTypePtr nodeType);
 
 #endif
