@@ -1,7 +1,7 @@
 /**
  * @file feed_list_view.c  the feed list in a GtkTreeView
  *
- * Copyright (C) 2004-2011 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2004-2013 Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  * Copyright (C) 2005 Raphael Slinckx <raphael@slinckx.net>
  * 
@@ -37,9 +37,8 @@
 #include "ui/browser_tabs.h"
 #include "ui/liferea_shell.h"
 #include "ui/subscription_dialog.h"
-#include "ui/ui_common.h"
 #include "ui/ui_dnd.h"
-#include "ui/ui_node.h"
+#include "ui/feed_list_node.h"
 #include "fl_sources/node_source.h"
 
 GtkTreeModel		*filter = NULL;
@@ -54,7 +53,7 @@ feed_list_view_row_changed_cb (GtkTreeModel *model, GtkTreePath *path, GtkTreeIt
 	
 	gtk_tree_model_get (model, iter, FS_PTR, &node, -1);
 	if (node)
-		ui_node_update_iter(node->id, iter);
+		feed_list_node_update_iter(node->id, iter);
 }
 
 static void
@@ -71,8 +70,12 @@ feed_list_view_selection_changed_cb (GtkTreeSelection *selection, gpointer data)
 
 		browser_tabs_show_headlines ();		// FIXME: emit signal to item list instead of bother the tabs manager
 		
-		/* update feed list and item list states */
+		/* 1.) update feed list and item list states */
 		feedlist_selection_changed (node);
+
+		/* 2.) Refilter the GtkTreeView to get rid of nodes with 0 unread 
+		       messages when in reduced mode. */
+		gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (filter));
 		
 		if (node) {
 			gboolean allowModify = (NODE_SOURCE_TYPE (node->source->root)->capabilities & NODE_SOURCE_CAPABILITY_WRITABLE_FEEDLIST);
@@ -110,7 +113,7 @@ feed_list_view_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer dat
 {
 	if ((event->type == GDK_KEY_PRESS) &&
 	    (event->state == 0) &&
-	    (event->keyval == GDK_Delete)) {
+	    (event->keyval == GDK_KEY_Delete)) {
 		nodePtr node = feedlist_get_selected ();
 				
 		if(node) {
@@ -140,6 +143,15 @@ feed_list_view_filter_visible_function (GtkTreeModel *model, GtkTreeIter *iter,
 	if (IS_FOLDER (node) || IS_NODE_SOURCE (node))
 		return FALSE;
 
+	if (IS_VFOLDER (node))
+		return TRUE;
+
+	/* Do not hide in any case if the node is selected, otherwise
+	   the last unread item of a feed causes the feed to vanish 
+	   when clicking it */
+	if (feedlist_get_selected () == node)
+		return TRUE;
+
 	if (count > 0)
 		return TRUE;
 
@@ -152,7 +164,7 @@ feed_list_view_expand (nodePtr node)
 	if (node->parent)
 		feed_list_view_expand (node->parent);
 
-	ui_node_set_expansion (node, TRUE);
+	feed_list_node_set_expansion (node, TRUE);
 }
 
 static void
@@ -190,7 +202,7 @@ feed_list_view_set_reduce_mode (gboolean newReduceMode)
 	feedlist_reduced_unread = newReduceMode;
 	conf_set_bool_value (REDUCED_FEEDLIST, feedlist_reduced_unread);
 	feed_list_view_reduce_mode_changed ();
-	ui_node_reload_feedlist ();
+	feed_list_node_reload_feedlist ();
 }
 
 static gint
@@ -206,7 +218,7 @@ void
 feed_list_view_sort_folder (nodePtr folder)
 {
 	folder->children = g_slist_sort (folder->children, feed_list_view_sort_folder_compare);
-	ui_node_reload_feedlist ();
+	feed_list_node_reload_feedlist ();
 	feedlist_foreach (feed_list_view_restore_folder_expansion);
 	feedlist_schedule_save ();
 }
@@ -216,9 +228,9 @@ feed_list_view_sort_folder (nodePtr folder)
 void
 feed_list_view_init (GtkTreeView *treeview)
 {
-	GtkCellRenderer		*textRenderer;
+	GtkCellRenderer		*titleRenderer, *countRenderer;
 	GtkCellRenderer		*iconRenderer;	
-	GtkTreeViewColumn 	*column;
+	GtkTreeViewColumn 	*column, *column2;
 	GtkTreeSelection	*select;	
 	
 	debug_enter ("feed_list_view_init");
@@ -228,7 +240,8 @@ feed_list_view_init (GtkTreeView *treeview)
 	                                G_TYPE_STRING,
 	                                GDK_TYPE_PIXBUF,
 	                                G_TYPE_POINTER,
-	                                G_TYPE_UINT);
+	                                G_TYPE_UINT,
+					G_TYPE_STRING);
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL (feedstore));
 
@@ -241,22 +254,31 @@ feed_list_view_init (GtkTreeView *treeview)
 
 	g_signal_connect (G_OBJECT (feedstore), "row-changed", G_CALLBACK (feed_list_view_row_changed_cb), NULL);
 
-	/* we only render the state and title */
+	/* we render the icon/state, the feed title and the unread count */
 	iconRenderer = gtk_cell_renderer_pixbuf_new ();
-	textRenderer = gtk_cell_renderer_text_new ();
+	titleRenderer = gtk_cell_renderer_text_new ();
+	countRenderer =  gtk_cell_renderer_text_new ();
+
+	gtk_cell_renderer_set_alignment (countRenderer, 1.0, 0);
 
 	column = gtk_tree_view_column_new ();
+	column2 = gtk_tree_view_column_new ();
 	
 	gtk_tree_view_column_pack_start (column, iconRenderer, FALSE);
-	gtk_tree_view_column_pack_start (column, textRenderer, TRUE);
+	gtk_tree_view_column_pack_start (column, titleRenderer, TRUE);
+	gtk_tree_view_column_pack_end (column2, countRenderer, FALSE);
 	
 	gtk_tree_view_column_add_attribute (column, iconRenderer, "pixbuf", FS_ICON);
-	gtk_tree_view_column_add_attribute (column, textRenderer, "markup", FS_LABEL);
-	
-	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_add_attribute (column, titleRenderer, "markup", FS_LABEL);
+	gtk_tree_view_column_add_attribute (column2, countRenderer, "markup", FS_COUNT);
+
+	gtk_tree_view_column_set_expand (column, TRUE);	
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_sizing (column2, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_append_column (treeview, column);
+	gtk_tree_view_append_column (treeview, column2);
 	
-	g_object_set (textRenderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+	g_object_set (titleRenderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 
 	g_signal_connect (G_OBJECT (treeview), "row-activated", G_CALLBACK (feed_list_view_row_activated_cb), NULL);
 	g_signal_connect (G_OBJECT (treeview), "key-press-event", G_CALLBACK (feed_list_view_key_press_cb), NULL);
@@ -275,7 +297,7 @@ feed_list_view_init (GtkTreeView *treeview)
 	ui_dnd_setup_feedlist (feedstore);
 	liferea_shell_update_feed_menu (TRUE, FALSE, FALSE);
 	liferea_shell_update_allitems_actions (FALSE, FALSE);
-	
+
 	debug_exit ("feed_list_view_init");
 }
 
@@ -284,9 +306,7 @@ feed_list_view_select (nodePtr node)
 {
 	GtkTreeView		*treeview;
 	GtkTreeModel		*model;
-	GtkWindow		*mainwindow;
 	
-	mainwindow = GTK_WINDOW (liferea_shell_get_window ());
 	treeview = GTK_TREE_VIEW (liferea_shell_lookup ("feedlist"));
 	model = gtk_tree_view_get_model (treeview);
 	
@@ -296,10 +316,10 @@ feed_list_view_select (nodePtr node)
 		/* in filtered mode we need to convert the iterator */
 		if (feedlist_reduced_unread) {
 			GtkTreeIter iter;
-			gtk_tree_model_filter_convert_child_iter_to_iter (GTK_TREE_MODEL_FILTER (filter), &iter, ui_node_to_iter (node->id));
+			gtk_tree_model_filter_convert_child_iter_to_iter (GTK_TREE_MODEL_FILTER (filter), &iter, feed_list_node_to_iter (node->id));
 			path = gtk_tree_model_get_path (model, &iter);
 		} else {
-			path = gtk_tree_model_get_path (model, ui_node_to_iter (node->id));
+			path = gtk_tree_model_get_path (model, feed_list_node_to_iter (node->id));
 		}
 		
 		if (node->parent)
@@ -412,12 +432,6 @@ on_menu_allfeedsread (GtkWidget *widget, gpointer user_data)
 	feedlist_mark_all_read (feedlist_get_root ());
 }
 
-void
-on_newbtn_clicked (GtkButton *button, gpointer user_data)
-{	
-	node_type_request_interactive_add (feed_get_node_type ());
-}
-
 void
 on_menu_feed_new (GtkMenuItem *menuitem, gpointer user_data)
 {
