@@ -1,7 +1,7 @@
-/**
+/*
  * @file itemview.c  viewing feed content in different presentation modes
  * 
- * Copyright (C) 2006-2011 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2012 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,11 +25,12 @@
 #include "debug.h"
 #include "folder.h"
 #include "htmlview.h"
+#include "item_history.h"
 #include "itemlist.h"
 #include "itemview.h"
 #include "node.h"
-#include "vfolder.h"
 #include "ui/ui_common.h"
+#include "ui/enclosure_list_view.h"
 #include "ui/liferea_shell.h"
 #include "ui/item_list_view.h"
 #include "ui/liferea_htmlview.h"
@@ -47,7 +48,8 @@
 struct ItemViewPrivate {
 	gboolean	htmlOnly;		/**< TRUE if HTML only mode */
 	guint		mode;			/**< current item view mode */
-	nodePtr		node;			/**< the node whose item are displayed */
+	nodePtr		node;			/**< the node whose items are displayed */
+	gboolean	browsing;		/**< TRUE if itemview is used as internal browser right now */
 	gboolean	needsHTMLViewUpdate;	/**< flag to be set when HTML rendering is to be 
 						     updated, used to delay HTML updates */
 	gboolean	hasEnclosures;		/**< TRUE if at least one item of the current itemset has an enclosure */
@@ -109,12 +111,14 @@ itemview_clear (void)
 	enclosure_list_view_hide (itemview->priv->enclosureView);
 	itemview->priv->hasEnclosures = FALSE;
 	itemview->priv->needsHTMLViewUpdate = TRUE;
+	itemview->priv->browsing = FALSE;
 }
 
 void
 itemview_set_mode (itemViewMode mode)
 {
 	if (itemview->priv->mode != mode) {
+		/* FIXME: Not being able to call itemview_clear() here is awful! */
 		itemview->priv->mode = mode;
 		htmlview_clear ();	/* drop HTML rendering cache */
 	}
@@ -170,8 +174,13 @@ void
 itemview_select_item (itemPtr item)
 {
 	ItemViewPrivate *ivp = itemview->priv;
-		
+
+	/* Enforce single item mode as we currently know no other way
+	   to select a single item... */
+	itemview_set_mode (ITEMVIEW_SINGLE_ITEM);
+
 	ivp->needsHTMLViewUpdate = TRUE;
+	ivp->browsing = FALSE;
 	
 	item_list_view_select (ivp->itemListView, item);
 	htmlview_select_item (item);
@@ -180,6 +189,14 @@ itemview_select_item (itemPtr item)
 		enclosure_list_view_load (ivp->enclosureView, item);
 	else
 		enclosure_list_view_hide (ivp->enclosureView);
+
+	item_history_add (item->id);
+}
+
+void
+itemview_select_enclosure (guint position)
+{
+	enclosure_list_view_select (itemview->priv->enclosureView, position);
 }
 
 void
@@ -189,6 +206,10 @@ itemview_update_item (itemPtr item)
 	if (ITEMVIEW_ALL_ITEMS != itemview->priv->mode)
 		item_list_view_update_item (itemview->priv->itemListView, item);
 
+	/* Bail if we do internal browsing, and no item is shown */
+	if (itemview->priv->browsing)
+		return;
+
 	/* Bail out if no HTML update necessary */
 	switch (itemview->priv->mode) {
 		case ITEMVIEW_ALL_ITEMS:
@@ -217,7 +238,7 @@ itemview_update_all_items (void)
 	/* Always update the GtkTreeView (bail-out done in ui_itemlist_update_item() */
 	if (ITEMVIEW_ALL_ITEMS != itemview->priv->mode)
 		item_list_view_update_all_items (itemview->priv->itemListView);
-		
+
 	itemview->priv->needsHTMLViewUpdate = TRUE;
 	htmlview_update_all_items ();
 }
@@ -225,6 +246,10 @@ itemview_update_all_items (void)
 void
 itemview_update_node_info (nodePtr node)
 {
+	/* Bail if we do internal browsing, and no item is shown */
+	if (itemview->priv->browsing)
+		return;
+
 	if (!itemview->priv->node)
 		return;
 	
@@ -252,6 +277,7 @@ itemview_update (void)
 		itemview->priv->needsHTMLViewUpdate = FALSE;
 		htmlview_update (itemview->priv->htmlview, itemview->priv->mode);
 	}
+
 	if (itemview->priv->node)
 		liferea_shell_update_allitems_actions (0 != itemview->priv->node->itemCount, 0 != itemview->priv->node->unreadCount);
 }
@@ -376,7 +402,6 @@ itemview_set_layout (nodeViewType newMode)
 			ilWidgetName = NULL;
 			encViewVBoxName = NULL;
 			break;
-		case NODE_VIEW_MODE_INVALID:
 		default:
 			g_warning("fatal: illegal viewing mode!");
 			return;
@@ -414,7 +439,7 @@ itemview_create (GtkWidget *window)
 	
 	g_object_new (ITEMVIEW_TYPE, NULL);
 	
-	itemview->priv->currentLayoutMode = NODE_VIEW_MODE_INVALID;
+	itemview->priv->currentLayoutMode = NODE_VIEW_MODE_DEFAULT;
 	itemview->priv->itemListView = item_list_view_create (window);
 	itemview->priv->itemListViewContainer = gtk_widget_get_parent (GTK_WIDGET (item_list_view_get_widget (itemview->priv->itemListView)));
 	conf_get_int_value (LAST_ZOOMLEVEL, &zoom);
@@ -435,18 +460,13 @@ itemview_create (GtkWidget *window)
 	return itemview;
 }
 
-GtkStyle *
-itemview_get_style (void)
-{
-	return gtk_widget_get_style (liferea_htmlview_get_widget (itemview->priv->htmlview));
-}
-
 void
 itemview_launch_URL (const gchar *url, gboolean forceInternal)
 {
 	gboolean internal;
 	
 	if (forceInternal) {
+		itemview->priv->browsing = TRUE;
 		liferea_htmlview_launch_URL_internal (itemview->priv->htmlview, url);
 		return;
 	}
@@ -461,6 +481,9 @@ itemview_launch_URL (const gchar *url, gboolean forceInternal)
 void
 itemview_do_zoom (gboolean in)
 {
-	g_assert(itemview->priv->htmlview != NULL);
+	if (itemview->priv->htmlview == NULL)
+		return;
+
 	liferea_htmlview_do_zoom (itemview->priv->htmlview, in);
 }
+
