@@ -1,7 +1,7 @@
 /**
  * @file update.c  generic update request and state processing
  *
- * Copyright (C) 2003-2008 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2014 Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  * Copyright (C) 2009 Adrian Bunk <bunk@users.sourceforge.net>
  *
@@ -28,17 +28,20 @@
 #include <libxslt/transform.h>
 #include <libxslt/xsltutils.h>
 
+#include <libpeas/peas-extension-set.h>
+
 #include <unistd.h>
 #include <stdio.h>
 #include <sys/wait.h>
 #include <string.h>
 
+#include "auth_activatable.h"
 #include "common.h"
 #include "debug.h"
 #include "net.h"
+#include "plugins_engine.h"
 #include "xml.h"
 #include "ui/liferea_shell.h"
-#include "ui/ui_tray.h"
 
 /** global update job list, used for lookups when cancelling */
 static GSList	*jobs = NULL;
@@ -68,6 +71,21 @@ update_state_set_lastmodified (updateStatePtr state, glong lastModified)
 	state->lastModified = lastModified;
 }
 
+const gchar *
+update_state_get_etag (updateStatePtr state)
+{
+	return state->etag;
+}
+
+void
+update_state_set_etag (updateStatePtr state, const gchar *etag)
+{
+	g_free (state->etag);
+	state->etag = NULL;
+	if (etag)
+		state->etag = g_strdup(etag);
+}
+
 const gchar *
 update_state_get_cookies (updateStatePtr state)
 {
@@ -91,6 +109,7 @@ update_state_copy (updateStatePtr state)
 	newState = update_state_new ();
 	update_state_set_lastmodified (newState, update_state_get_lastmodified (state));
 	update_state_set_cookies (newState, update_state_get_cookies (state));
+	update_state_set_etag (newState, update_state_get_etag (state));
 	
 	return newState;
 }
@@ -102,6 +121,7 @@ update_state_free (updateStatePtr updateState)
 		return;
 
 	g_free (updateState->cookies);
+	g_free (updateState->etag);
 	g_free (updateState);
 }
 
